@@ -1,6 +1,5 @@
-use zellij_tile::prelude::Style;
 mod floating_pane_grid;
-use zellij_utils::{position::Position, zellij_tile};
+use zellij_utils::position::Position;
 
 use crate::tab::Pane;
 use floating_pane_grid::FloatingPaneGrid;
@@ -16,8 +15,10 @@ use std::cell::RefCell;
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::rc::Rc;
 use std::time::Instant;
-use zellij_tile::data::ModeInfo;
-use zellij_utils::pane_size::{Offset, PaneGeom, Size, Viewport};
+use zellij_utils::{
+    data::{ModeInfo, Style},
+    pane_size::{Offset, PaneGeom, Size, Viewport},
+};
 
 macro_rules! resize_pty {
     ($pane:expr, $os_input:expr) => {
@@ -486,7 +487,7 @@ impl FloatingPanes {
 
                     self.set_force_render();
                     return true;
-                }
+                },
                 None => Some(active_pane_id),
             }
         } else {
@@ -500,12 +501,12 @@ impl FloatingPanes {
                 }
                 self.set_pane_active_at(updated_active_pane);
                 self.set_force_render();
-            }
+            },
             None => {
                 // TODO: can this happen?
                 self.active_panes.clear();
                 self.z_indices.clear();
-            }
+            },
         }
         false
     }
@@ -557,7 +558,7 @@ impl FloatingPanes {
                     self.set_pane_active_at(p);
                     self.set_force_render();
                     return true;
-                }
+                },
                 None => Some(active_pane_id),
             }
         } else {
@@ -571,12 +572,12 @@ impl FloatingPanes {
                 }
                 self.set_pane_active_at(updated_active_pane);
                 self.set_force_render();
-            }
+            },
             None => {
                 // TODO: can this happen?
                 self.active_panes.clear();
                 self.z_indices.clear();
-            }
+            },
         }
         false
     }
@@ -627,7 +628,7 @@ impl FloatingPanes {
                     self.set_force_render();
                     self.set_pane_active_at(p);
                     return true;
-                }
+                },
                 None => Some(active_pane_id),
             }
         } else {
@@ -641,12 +642,12 @@ impl FloatingPanes {
                 }
                 self.set_pane_active_at(updated_active_pane);
                 self.set_force_render();
-            }
+            },
             None => {
                 // TODO: can this happen?
                 self.active_panes.clear();
                 self.z_indices.clear();
-            }
+            },
         }
         false
     }
@@ -697,7 +698,7 @@ impl FloatingPanes {
 
                     self.set_force_render();
                     return true;
-                }
+                },
                 None => Some(active_pane_id),
             }
         } else {
@@ -711,12 +712,12 @@ impl FloatingPanes {
                 }
                 self.set_pane_active_at(updated_active_pane);
                 self.set_force_render();
-            }
+            },
             None => {
                 // TODO: can this happen?
                 self.active_panes.clear();
                 self.z_indices.clear();
-            }
+            },
         }
         false
     }
@@ -789,10 +790,10 @@ impl FloatingPanes {
                     Some(next_active_pane) => {
                         self.active_panes.insert(client_id, next_active_pane);
                         self.focus_pane(next_active_pane, client_id);
-                    }
+                    },
                     None => {
                         self.defocus_pane(pane_id, client_id);
-                    }
+                    },
                 }
             }
         }
